#ifndef _PKTIF_CALLBACKMGR_H
#define _PKTIF_CALLBACKMGR_H

#include "util/cyc_queue.h"
#include <pktif.h>

#define MAX_SKTID 256

#define CB_TYPE_MSG 0
#define CB_TYPE_NDATA 1
#define CB_TYPE_NTAG 2
#define CB_TYPE_NSTAT 3
#define CB_TYPE_NDROP 4
#define CB_TYPE_MAX 5

#define PKTLAB_TCPEV_MAX (PKTLAB_TCPEV_PEERCLOSED+1)

struct general_cb {
    uint8_t type;
    bool installed;
    bool discard;
    pktif_cb_t cb;

    union {
        struct {
            enum pktlab_message_type req_type;
        } msg;

        struct {
            uint8_t sktid;
        } ndata;

        struct {
            uint16_t tag;
        } ntag;

        struct {
            uint8_t sktid;
            uint8_t tcpev;
        } nstat;

        struct {
            uint8_t sktid;
        } ndrop;
    };
};

struct callbackmgr {
    struct pktif *pktif;
    // some circular queue buffer for callback functions
    struct cyc_queue *msg_cb;

    // callback functions for NDATA notifications
    struct general_cb ndata_cb[MAX_SKTID];

    // callback functions for NSTAT notifications
    struct general_cb nstat_cb[MAX_SKTID][PKTLAB_TCPEV_MAX];

    struct general_cb ntag_cb[UINT16_MAX];

    struct general_cb ndrop_cb[MAX_SKTID];

    struct cyc_queue *data_buf[MAX_SKTID];

    // shared between pktif and callback mgr
    // pktif is responsible for creating the guard
    // TODO: MT-Safe version
    // struct ser_guard *guard;
};

struct callbackmgr *callbackmgr_create(struct pktif *pktif);
int callbackmgr_teardown(struct callbackmgr *mgr);

// register callback, synchronously delivered
int register_msg_cb(struct callbackmgr *mgr, pktif_cb_t *cb,
                    enum pktlab_message_type req_type);

#define PKTIF_CB_CTL_ADD 1
#define PKTIF_CB_CTL_DEL 2
#define PKTIF_CB_CTL_MOD 3

// register event callback, asynchronously delivered
int general_cb_ctl(struct callbackmgr *mgr, int op, struct general_cb *cbptr);
// TODO: expose the notification ctl
int notif_cb_ctl(struct callbackmgr *mgr, int op, uint8_t type, bool discard,
                 uint8_t sktid, uint16_t tag, uint8_t tcpev, pktif_cb_t *cb);

int invoke_cb(struct callbackmgr *mgr, struct pktlab_message *msg);

int invoke_msg_cb(struct callbackmgr *mgr, struct pktlab_message *msg);

#endif