#include "cyc_queue.h"
#include "debug.h"
#include <errno.h>
#include <string.h>
struct cyc_queue *cyc_queue_init(size_t arr_size) {
    if (arr_size == 0 || ((arr_size - 1) & arr_size) != 0) {
        return NULL;
    }

    struct cyc_queue *cq = malloc(sizeof(*cq));
    if (!cq)
        return NULL;

    cq->arr = malloc(sizeof(*cq->arr) * arr_size);
    if (!cq->arr) {
        free(cq);
        return NULL;
    }

    cq->head = 0;
    cq->tail = arr_size - 1;
    cq->size = arr_size;
    cq->is_empty = true;
    return cq;
}

void cyc_queue_destroy(struct cyc_queue *cq) {
    if (!cq)
        return;
    free(cq->arr);
    free(cq);
}

int cyc_queue_push(struct cyc_queue *cq, void *elem) {
    if (!cq)
        return -1;

    if (cq->is_empty) {
        cq->is_empty = false;
        cq->tail = cq->head;
        cq->arr[cq->tail] = elem;
        return 0;
    }

    if (cyc_queue_length(cq) >= cq->size) {
        // reallocate
        void **new_arr = realloc(cq->arr, sizeof(*cq->arr) * cq->size * 2);
        if (new_arr == NULL) {
            errno = ENOMEM;
            return -1;
        }
        cq->arr = new_arr;
        if (cq->tail < cq->head) {
            memcpy(cq->arr + cq->size, cq->arr, cq->tail * (sizeof(*cq->arr)));
            cq->tail += cq->size;
        }
        cq->size *= 2;
    }

    cq->tail = (cq->tail + 1) % cq->size;
    cq->arr[cq->tail] = elem;
    return 0;
}

void *cyc_queue_peek_head(struct cyc_queue *cq) {
    if (!cq || cq->is_empty) {
        return NULL;
    }

    return cq->arr[cq->head];
}

void *cyc_queue_pop(struct cyc_queue *cq) {
    void *head = cyc_queue_peek_head(cq);
    size_t len = cyc_queue_length(cq);
    if (head) {
        cq->head = (cq->head + 1) % cq->size;
        if (len == 1)
            cq->is_empty = true;
    }
    return head;
}

size_t cyc_queue_length(struct cyc_queue *cq) {
    if (!cq || cq->is_empty)
        return 0;

    // +cq->size is necesary because we are operating on unsigned numbers
    // this is needed for correctness
    return (cq->tail + cq->size - cq->head) % cq->size + 1;
}
