#ifndef _PKTIF_UTIL_DEBUG_H
#define _PKTIF_UTIL_DEBUG_H
#include <pktlab.h>
#include <stddef.h>

#define DEBUG_LABEL "DEBUG"
#define TRACE_LABEL "TRACE"
#define FATAL_LABEL "FATAL"
#define WARN_LABEL "WARN"
#define INFO_LABEL "INFO"
#define ERROR_LABEL "ERROR"

// #define debug(...) debug_print(__FILE__, __LINE__, DEBUG_LABEL, __VA_ARGS__)

// #define trace(...) debug_print(__FILE__, __LINE__, TRACE_LABEL, __VA_ARGS__)

#ifdef DEBUG
#define debug(...) debug_print(__FILE__, __LINE__, DEBUG_LABEL, __VA_ARGS__)
#else
#define debug(...)                                                             \
    do {                                                                       \
    } while (0)
#endif

extern void pktlab_debug_(const char *restrict filename, int lineno,
                          const char *restrict fmt, ...);

#ifdef TRACE
#define trace(...) debug_print(__FILE__, __LINE__, TRACE_LABEL, __VA_ARGS__)
#else
#define trace(...)                                                             \
    do {                                                                       \
    } while (0)
#endif

#define fatal(...)                                                             \
    do {                                                                       \
        debug_print(__FILE__, __LINE__, FATAL_LABEL, __VA_ARGS__);             \
        exit(-1);                                                              \
    } while (0)
#define warn(...) debug_print(__FILE__, __LINE__, WARN_LABEL, __VA_ARGS__)

#define info(...) debug_print(__FILE__, __LINE__, INFO_LABEL, __VA_ARGS__)
#define error(...) debug_print(__FILE__, __LINE__, ERROR_LABEL, __VA_ARGS__)

void pl_msg_dump_(struct pktlab_message *msg, size_t size)
    __attribute__((unused));
void print_err_exit(int exit_val, const char *fmt, ...);
void perror_exit(const char *str, int exit_val);
void print_backtrace();
void debug_print(const char *restrict filename, int lineno,
                 const char *restrict label, const char *restrict fmt, ...);
#endif