// prog_openssl.c
//

#include <inttypes.h>
#include <stdint.h>

#include <openssl/err.h>
#include <openssl/sha.h>

#include <private.h>
#include <util_openssl.h>

//
// EXPORTED FUNCTION DEFINITIONS
//

int pktlab_get_program_digest (
    const void * data, uint_fast32_t datalen, uint8_t * digest)
{
    trace("%s(buflen:%u)", __func__, (unsigned) datalen);

    if (data == NULL || datalen == 0 || digest == NULL) {
        debug("%s(data:%p,datalen:%u,digest:%p): bad argument",
            __func__, data, (unsigned) datalen, digest);
        return -1;
    }

    if (datalen > SIZE_MAX) {
        debug("%s(datalen:%" PRIuFAST32 "): overflow",
            __func__, datalen);
        return -2;
    }

    ERR_clear_error();
    if (SHA256(data, datalen, digest) == NULL) {
        error_debug_msg("SHA256");
        return -3;
    }

    return 0;
}