// config.h
//

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#include <pktlab.h>
#include <pktconf.h>

//
// EXPORTED CONSTANTS
//

enum config_st {
    CONFIG_NOT_INIT = 0,
    CONFIG_INIT     = 1,
    CONFIG_READY    = 2
};

//
// EXPORTED TYPE DEFINITIONS
//

struct exp_exec_entry {
    bool is_default;
    const char * sni_key;
    const struct pktconf_sep_str_ll * exppriv_chain_path_ll;
    const struct pktconf_sep_str_ll * trusted_eop_path_ll;
    const struct pktconf_sep_str_ll * pub_channel_ll;
    const char * override_xdescr;
    const struct pktlab_xdescr_dattr_pair * override_xdescr_nvps;
    uint_fast32_t override_xdescr_nvpcnt;
    uint_fast32_t lineno;
    const struct exp_exec_entry * next;
};

struct config {
    int state;
    bool enable_exp_pub_specified;
    bool enable_exp_pub;
    const char * broker_addr;
    const char * pub_controller_addr;
    const char * pub_agent_cert_path;
    const struct pktconf_sep_str_ll * broker_cert_root_path_ll;
    bool pub_period_specified;
    pktlab_time_t pub_period;
    bool pub_exp_priority_specified;
    unsigned long long pub_exp_priority;
    const struct pktconf_sep_str_ll * pub_country;
    const struct pktconf_sep_str_ll * pub_location;
    const char * controller_privkey_path;
    const char * program_dir_path;
    bool pub_end_greet_period_specified;
    pktlab_time_t pub_end_greet_period;
    bool enable_proxy_mode_specified;
    bool enable_proxy_mode;
    const char * backlog_conn_agent_cert_path;
    const struct pktconf_sep_str_ll * proxy_cert_root_path_ll;
    bool target_backlog_conn_specified;
    unsigned long long target_backlog_conn;
    bool backlog_setup_timeout_specified;
    pktlab_time_t backlog_setup_timeout;
    const struct exp_exec_entry * exp_exec_ll;
};

//
// EXPORTED GLOBAL VARIABLES
//

extern struct config _config;

//
// EXPORTED FUNCTION DECLARATIONS
//

extern int config_init(void);
extern int config_read(const char * path, uint_fast32_t * errlineno);
extern void config_cleanup(void);

extern bool is_config_presence_correct(int * err);

#endif
