// console.c
//

#include "console.h"

#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

#include <pktlab.h>

//
// EXPORTED FUNCTION DEFINITIONS
//

void pktxpmgr_print_ (
    const char * filename, int lineno, const char * label, const char * fmt, ...)
{
    char buf[4096]; // potential truncation
    va_list args;

    va_start(args, fmt);
    vsnprintf(buf, sizeof(buf), fmt, args);
    va_end(args);

    fprintf(stderr, "%s (%s:%d): %s\n", label, filename, lineno, buf);
    fflush(stderr);
}

char * chlist2str (
    const void * chlistptr, uint_fast32_t chlistlen)
{
    static char buf[4096] = {}; // potential truncation
    struct pktlab_channel * channels;
    struct pktlab_channel * ch;
    uint8_t * ptr;
    uint_fast32_t i;
    int rst;
    size_t len;

    if (chlistptr == NULL || chlistlen == 0)
        return buf;

    if (pktlab_decode_chlist(chlistptr, chlistlen, &channels) < 0)
        return buf; // return empty string if things went wrong

    for (len = 0, ch = channels; ch != NULL; ch = ch->next) {
        ptr = ch->channel;
        for (i = 0; i < ch->channellen; ++i) {
            rst = snprintf(
                buf+len, sizeof(buf)-len, "%s%02x",
                (i == 0 && ch != channels) ? ",":"", ptr[i]);
            if (rst < 0) {
                buf[0] = '\0';
                goto fail;
            }

            if (rst > sizeof(buf)-1-len)
                goto trunc;

            len += rst;
        }
    }

trunc:
fail:
    pktlab_free_channels(channels);
    return buf;
}