// ge_deputy.h
// Common interface for greet endpoint deputy module
//

#ifndef _GE_DEPUTY_H_
#define _GE_DEPUTY_H_

#include <pktctrl.h>
#include <pktlab.h>

#include <stdint.h>

#include <netdb.h>
#include <sys/select.h>

#include "snikey.h"

//
// EXPORTED TYPE DEFINITIONS
//

struct ge_passive_aux {
    struct addrinfo * listen_ai;
};

struct ge_active_aux {
    struct addrinfo * proxy_ai;
    struct pktlab_certificate * agent_cert;
    struct pktlab_publickey ** trusted_proxy_root;
    uint_fast32_t rootno;
    uint_fast32_t target_backlog;
    struct snikey_ll_node * snikeys;
    pktlab_time_t proxy_timeout;
};

typedef int (*ge_deputy_init_t) (
    char ** exp_prog_n_args,
    struct pktlab_privatekey * controller_key,
    struct pktctrl_keycert_store * greet_store,
    void * aux);
typedef int (*ge_deputy_start_t) (void);

typedef int (*ge_deputy_prepare_select_t) (
    int * nfds, fd_set * rset, fd_set * wset,
    pktlab_time_t * select_until);
typedef int (*ge_deputy_process_select_t) (
    const fd_set * rset, const fd_set * wset,
    pktlab_time_t now);

typedef int (*ge_deputy_wrapup_t) (void);
typedef int (*ge_deputy_cleanup_t) (void);

typedef bool (*ge_deputy_is_done_t) (void);

struct ge_methods {
    ge_deputy_init_t init;
    ge_deputy_start_t start;
    ge_deputy_prepare_select_t prepare_select;
    ge_deputy_process_select_t process_select;
    ge_deputy_wrapup_t wrapup;
    ge_deputy_cleanup_t cleanup;
    ge_deputy_is_done_t is_done;
};

//
// EXPORTED FUNCTION DECLARATIONS
//

extern struct ge_methods get_passive_ge_methods(void);
extern struct ge_methods get_active_ge_methods(void);

#endif