// comm.c
//

#include <assert.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>

#include "comm.h"
#include "console.h"

int recv_ctrlmsg(int fd, struct ctrlmsg *msg) {
    char *buf;
    int size, off = 0;

    assert(msg != NULL);

    trace("%s(fd:%d)", __func__, fd);

    buf = (char *)msg;
    while (off < sizeof(*msg)) {
        size = read(fd, buf + off, sizeof(*msg) - off);
        if (size < 0) {
            debug("%s read failure errno(%s)\n", __func__, strerror(errno));
            return -1;
        } else if (size == 0)
            return 1; // EOF detected

        off += size;
    }

    return 0;
}

int send_ctrlmsg(int fd, enum ctrlmsg_type type, pktlab_time_t yield_hint) {
    struct ctrlmsg msg_send;
    int off = 0;
    int size;
    trace("%s fd(%d), type(%d)", __func__, fd, type);
    if (type <= CTRLMSG_TYPE_UNDEF || type >= CTRLMSG_TYPE_NUM) {
        warn("invalid type");
        return -1;
    }
    msg_send.type = type;
    if (type == CTRLMSG_TYPE_YIELD) {
        msg_send.yield.time = yield_hint;
    }
    while (off < sizeof(msg_send)) {
        size = write(fd, (char *)&msg_send + off, sizeof(msg_send) - off);
        if (size < 0)
            fatal("%s write failure %s\n", __func__, strerror(errno));
        off += size;
    }
    return 0;
}
