// control.h
//

#ifndef _CONTROL_H_
#define _CONTROL_H_

#include <stdbool.h>
#include <stdint.h>
#include <sys/time.h>
#include <sys/select.h>
#include <pktlab.h>
#include <pktctrl.h>

#include "raw.h"

//
// EXPORTED TYPE DEFINITIONS
//

typedef void (*send_msg_cb_t) (void * aux);

//
// EXPORTED FUNCTION DECLARATIONS
//

extern void control_initialize(void);
extern void control_start(const char *ctlspec, const char *snikey, int mgrfd,
                          uint_fast32_t sleep_sec, uint_fast32_t sleep_timeout,
                          struct pktctrl_keycert_store *store,
                          struct pktlab_privatekey *privk,
                          uint32_t exp_priority);
extern void control_reset(void);
extern void control_suspend(void);
extern bool control_running(void);

// Control communication

// IMPORTANT: for control_push_msgq(_cb), msg needs to be FREEABLE!!
// cb called after msg flushed out
extern void control_push_msgq_cb(struct pktlab_message *msg,
                                 send_msg_cb_t cb, void * aux);
extern void control_push_msgq(struct pktlab_message *msg); // no cb called
extern void control_report(enum pktlab_status errid);      // special case for result msg

// Select handlers

extern int control_prepare_select(fd_set *restrict rset, fd_set *restrict wset,
                                  pktlab_time_t *select_until);

extern int control_process_select(int nsel, const fd_set *restrict rset,
                                  const fd_set *restrict wset);

// Program access handlers

extern struct raw_prog * control_fetch_cert_filters(void);
extern struct raw_prog * control_fetch_cert_send_monitors(void);
extern struct raw_prog * control_fetch_cert_recv_monitors(void);

#endif
