// limitations.h
//

#ifndef LIMITATIONS_H
#define LIMITATIONS_H

#include <stdbool.h>
#include <stdint.h>

#include <pktlab.h>

//
// EXPORTED MACROS
//

// Temporary hard-coded limits
#define LIMITS_MAX_EXP_PERIOD  10000000000 // 10 sec in pktlab ticks
#define LIMITS_MAX_SEND_BYTE   1000000     // 1 MB
#define LIMITS_MAX_SEND_PKT    10000
#define LIMITS_MAX_RECV_BYTE   10000000    // 10 MB
#define LIMITS_MAX_RECV_PKT    100000
#define LIMITS_BURST_RATE_BYTE 10000000
#define LIMITS_BURST_RATE_PKT  10000000
#define LIMITS_BURST_RATE_TIME 10

//
// EXPORTED TYPE DEFINITIONS
//

// tmp struct def, to be in pktlab.h
struct pktlab_valid_dest {
    int prfam;
    uint8_t ip_address[PKTLAB_ADDRLEN_MAX];
    uint8_t network_mask[PKTLAB_ADDRLEN_MAX];
    struct pktlab_valid_dest * next;
};

// linked list definition of nodes containing timestamp and data
struct burst_rate_data {
    uint64_t size;
    pktlab_time_t timestamp;
    struct burst_rate_data * next;
};

struct burst_rate_control {
    uint64_t cnt;                         // burst rate upperbound cnt (B/pkt)
    uint64_t wincnt;                      // actual cnt in window, should <= cnt
    pktlab_time_t win;                    // burst rate period in pktlab ticks
    struct burst_rate_data * burst_data;
    struct burst_rate_control * next;
};

//
// EXPORTED FUNCTION DECLARATIONS
//

// module state transition functions
//

extern void limitations_init(void);
extern void limitations_reset(void);

// export limitation value functions
//

extern uint64_t get_burst_rate_max_pkt_byte_len(void);

// check functions
//

extern bool check_socket_cnt(void);
extern bool check_exp_priority(uint32_t exp_priority);
extern bool check_exp_endtime(pktlab_time_t current_time);
extern bool check_src_limit(const uint8_t * src_ip, uint8_t prfam);
extern bool check_dst_limit(const uint8_t * dst_ip, uint8_t prfam);

extern bool check_send_limit (
    uint64_t size, pktlab_time_t cur_time,
    pktlab_time_t* wait_until_time);
extern bool check_recv_limit(uint64_t size);

extern int get_burst_wait_time (
    uint64_t data_size, pktlab_time_t current_time,
    pktlab_time_t* wait_until_time);

// update functions
//

extern void consume_socket_cnt(void);
extern void consume_send_quota(uint64_t size, pktlab_time_t cur);
extern void consume_recv_quota(uint64_t size);

#endif