// raw_linux.h
//

#ifndef _RAW_LINUX_H_
#define _RAW_LINUX_H_

// POSIX-compatible include headers
#include <errno.h>
#include <stdbool.h>
#include <stdint.h>

#include <pktlab.h>

#include "raw.h"

// Linux dependent headers
#include <linux/bpf.h>
#include <linux/filter.h>

//
// EXPORTED MACRO DEFINITIONS
//

#define IP4_ALEN PKTLAB_IPV4_WO_MSK_ADDR_LEN
#define IP6_ALEN PKTLAB_IPV6_ADDR_LEN
#define MAX_PKTSZ 0x10000

#define try_again(en) ((en) == EAGAIN || (en) == EWOULDBLOCK)

#define RAW_CBPF_INSTR_SZ sizeof(struct sock_filter)
#define RAW_EBPF_INSTR_SZ sizeof(struct bpf_insn)

//
// INTERNAL TYPE DEFINITIONS
//

struct ifindex_ll {
    int ifindex;
    struct ifindex_ll * next;
};

struct raw_socket_obj_aux {
    int ifindex;
    uint8_t intfaddr[PKTLAB_ADDRLEN_MAX];
};

//
// EXPORTED FUNCTION DECLARATIONS
//

extern int addr2ifindex (
    int af, const uint8_t* addr);

extern int safe_socket (
    int af, int type, int protocol,
    enum raw_error *err);

extern bool is_allowed_recv_flags(int flags);

#endif