// raw_lkm_dummy.c
//

#include <stdint.h>
#include <sys/types.h>

#include <pktlab.h>

#include "console.h"
#include "raw.h"

//
// INTERNAL FUNCTION DECLARATIONS
//

static int lkm_raw_socket_setup_v4 (
    const uint8_t * intfaddr,
    struct raw_prog * cert_prog_ll,
    struct raw_socket_obj * obj);

static ssize_t lkm_raw_socket_send (
    const void * buf, size_t len, int flags,
    struct raw_socket_obj * obj, enum raw_error * err);
static ssize_t lkm_raw_socket_recv (
    void * buf, size_t len, int flags,
    struct raw_socket_obj * obj, enum raw_error * err);

static int lkm_raw_socket_set_recvfilter (
    struct raw_prog * filter,
    struct raw_socket_obj * obj);

static void lkm_raw_socket_close(struct raw_socket_obj * obj);

//
// EXPORTED FUNCTION DEFINITIONS
//

const struct raw_socket_method_t * raw_socket_steal_method(void) {
    static const struct raw_socket_method_t ret = {
        .raw_socket_setup_v4 = lkm_raw_socket_setup_v4,
        .raw_socket_setup_v6 = NULL, // ipv6 not supported yet
        .raw_socket_close = lkm_raw_socket_close,
        .raw_socket_send = lkm_raw_socket_send,
        .raw_socket_recv = lkm_raw_socket_recv,
        .raw_socket_set_recvfilter = lkm_raw_socket_set_recvfilter,
    };

    return &ret;
}

//
// INTERNAL FUNCTION DEFINITIONS
//

int lkm_raw_socket_setup_v4 (
    const uint8_t * intfaddr,
    struct raw_prog * cert_prog_ll,
    struct raw_socket_obj * obj)
{
    trace("%s(intfaddr:%p,cert_prog_ll:%p)",
        __func__, intfaddr, cert_prog_ll);
    debug("%s not supported due to OS restriction", __func__);
    return RAW_UNKFAULT;
}

void lkm_raw_socket_close(struct raw_socket_obj * obj) {
    trace("%s(...)", __func__);
    debug("%s not supported due to OS restriction", __func__);
    return;
}

ssize_t lkm_raw_socket_send (
    const void * buf, size_t len, int flags,
    struct raw_socket_obj * obj, enum raw_error * err)
{
    trace("%s(len:%zu,flags:%d)",
        __func__, len, flags);
    debug("%s not supported due to OS restriction", __func__);
    if (err != NULL)
        *err = RAW_UNKFAULT;
    return -1;
}


ssize_t lkm_raw_socket_recv (
    void * buf, size_t len, int flags,
    struct raw_socket_obj * obj, enum raw_error * err)
{
    trace("%s(len:%zu,flags:%d)",
        __func__, len, flags);
    debug("%s not supported due to OS restriction", __func__);
    if (err != NULL)
        *err = RAW_UNKFAULT;
    return -1;
}

int lkm_raw_socket_set_recvfilter (
    struct raw_prog * filter,
    struct raw_socket_obj * obj)
{
    trace("%s(filter:%p)", __func__, filter);
    debug("%s not supported due to OS restriction", __func__);
    return RAW_UNKFAULT;
}