// rlim.h - Resource limits
// Kirill Levchenko

#ifndef _RLIM_H_
#define _RLIM_H_

#include <stdint.h>
#include <stdio.h>
#include <pktlab.h>

//
// EXPORTED TYPE DEFINITIONS
//

struct rlim_accs {
	uint64_t sndpktcnt; // number of send packets
	uint64_t rcvpktcnt; // number of recevied packets
	uint64_t sndbytecnt; // number of sent bytes
	uint64_t rcvbytecnt; // number of received bytes
	uint64_t runtime; // runtime (no need to update)
};

struct rlim_caps {
	uint16_t sktcnt; // number of sockets
};

struct rlim_rate {
	struct rlim_rate * next; // next record in linked list
	pktlab_time_t win; // sliding time window (duration)
	uint64_t cnt; // maximum count
};

struct rlim_rates {
	struct rlim_rate * sndpktlist; // max send packet burst
	struct rlim_rate * sndbytelist; // max send byte burst
};

enum rlim_state {
	RLIM_UNINIT,
	RLIM_READY,
	RLIM_RUNNING,
	RLIM_PAUSED
};

struct rlim {
	enum rlim_state state; // current state
	pktlab_time_t endtime; // includes runtime limit
	struct rlim_accs accs;
	struct rlim_caps caps;
	struct rlim_rates rates;
};

//
// EXPORTED VARIABLE DECLARATIONS
//

extern struct rlim rlim;

//
// EXPORTED FUNCTION DECLARATIONS
//

extern void rlim_init(const char * storepath);
extern void rlim_finalize();
extern void rlim_vacuum();

extern void rlim_begin (
	struct pktlab_certificate ** certs,
	uint_fast32_t certcnt,
	pktlab_time_t now);

extern void rlim_pause(pktlab_time_t now);
extern void rlim_resume(pktlab_time_t now);

extern void rlim_commit(pktlab_time_t now);
extern void rlim_reset();

extern void rlim_print(FILE * out);

extern const char * rlim_stname(enum rlim_state state);

#endif // _RLIM_H_
